@@ -217,13 +217,9 @@ static void
 libbalsa_message_body_set_text_rfc822headers(LibBalsaMessageBody *body)
 {
     GMimeStream *headers;
-    GError *err = NULL;
 
     libbalsa_mailbox_lock_store(body->message->mailbox);
-    headers = libbalsa_message_body_get_stream(body, &err);
-    if (err != NULL) {
-	g_error_free(err);
-    }
+    headers = libbalsa_message_body_get_stream(body, NULL);
 
     if (headers != NULL) {
 	GMimeMessage *dummy_msg;
@@ -584,21 +580,33 @@ libbalsa_message_body_get_stream(LibBalsaMessageBody * body, GError **err)
     g_return_val_if_fail(body->message != NULL, NULL);
 
     if (!body->message->mailbox) {
-        g_set_error(err, LIBBALSA_MAILBOX_ERROR,
-                    LIBBALSA_MAILBOX_ACCESS_ERROR,
-                    "Internal error in get_stream");
+        if (err != NULL && *err == NULL) {
+            g_set_error(err, LIBBALSA_MAILBOX_ERROR,
+                        LIBBALSA_MAILBOX_ACCESS_ERROR,
+                        "Internal error in get_stream");
+        }
         return NULL;
     }
 
     if (!libbalsa_mailbox_get_message_part(body->message, body, err)
-        || !(GMIME_IS_PART(body->mime_part)
-             || GMIME_IS_MESSAGE_PART(body->mime_part))) {
-        if (err && !*err)
+        || body->mime_part == NULL) {
+        if (err != NULL && *err == NULL) {
+            g_set_error(err, LIBBALSA_MAILBOX_ERROR,
+                        LIBBALSA_MAILBOX_ACCESS_ERROR,
+                        "Cannot get stream for part");
+        }
+        return NULL;
+    }
+
+    if (!(GMIME_IS_PART(body->mime_part)
+          || GMIME_IS_MESSAGE_PART(body->mime_part))) {
+        if (err != NULL && *err == NULL) {
             g_set_error(err, LIBBALSA_MAILBOX_ERROR,
                         LIBBALSA_MAILBOX_ACCESS_ERROR,
                         "Cannot get stream for part of type %s",
                         g_type_name(G_TYPE_FROM_INSTANCE
                                     (body->mime_part)));
+        }
         return NULL;
     }
 
@@ -872,22 +880,19 @@ libbalsa_message_body_get_by_id(LibBalsaMessageBody * body,
 LibBalsaMsgProtectState
 libbalsa_message_body_protect_state(LibBalsaMessageBody *body)
 {
-    if (!body || !body->sig_info ||
-	body->sig_info->status == GPG_ERR_NOT_SIGNED ||
-	body->sig_info->status == GPG_ERR_CANCELED)
-	return LIBBALSA_MSG_PROTECT_NONE;
+	LibBalsaMsgProtectState state;
 
-    if (body->sig_info->status == GPG_ERR_NO_ERROR) {
-	/* good signature, check if the validity and trust (OpenPGP only) are
-	   at least marginal */
-	if (body->sig_info->validity >= GPGME_VALIDITY_MARGINAL &&
-	    (body->sig_info->protocol == GPGME_PROTOCOL_CMS ||
-	     body->sig_info->key->owner_trust >= GPGME_VALIDITY_MARGINAL))
-	    return LIBBALSA_MSG_PROTECT_SIGN_GOOD;
-	else
-	    return LIBBALSA_MSG_PROTECT_SIGN_NOTRUST;
-    }
-    
-    return LIBBALSA_MSG_PROTECT_SIGN_BAD;
+	if ((body == NULL) || (body->sig_info == NULL) || (body->sig_info->status == GPG_ERR_NOT_SIGNED) ||
+		(body->sig_info->status == GPG_ERR_CANCELED)) {
+		state = LIBBALSA_MSG_PROTECT_NONE;
+	} else if (body->sig_info->status != GPG_ERR_NO_ERROR) {
+		state = LIBBALSA_MSG_PROTECT_SIGN_BAD;
+	} else if ((body->sig_info->summary & GPGME_SIGSUM_VALID) == GPGME_SIGSUM_VALID) {
+		state = LIBBALSA_MSG_PROTECT_SIGN_GOOD;
+	} else {
+		state = LIBBALSA_MSG_PROTECT_SIGN_NOTRUST;
+	}
+
+	return state;
 }
 #endif
