@@ -25,10 +25,19 @@
 #include <gpgme.h>
 #include <string.h>
 #include <glib.h>
+#include <glib/gi18n.h>
 #include "libbalsa-gpgme.h"
 #include "misc.h"
+#include "libbalsa.h"
+#include "libbalsa-gpgme-keys.h"
+#include "libbalsa-gpgme-widgets.h"
 #include "gmime-gpgme-signature.h"
 
+#ifdef G_LOG_DOMAIN
+#  undef G_LOG_DOMAIN
+#endif
+#define G_LOG_DOMAIN "crypto"
+
 
 /* stuff for the signature status as returned by gpgme as an GObject */
 static GObjectClass *g_mime_gpgme_sigstat_parent_class = NULL;
@@ -36,7 +45,6 @@ static GObjectClass *g_mime_gpgme_sigstat_parent_class = NULL;
 static void g_mime_gpgme_sigstat_class_init(GMimeGpgmeSigstatClass *
 					    klass);
 static void g_mime_gpgme_sigstat_finalize(GMimeGpgmeSigstat * self);
-static void g_mime_gpgme_sigstat_init(GMimeGpgmeSigstat * self);
 
 
 /* GMimeGpgmeSigstat related stuff */
@@ -55,7 +63,7 @@ g_mime_gpgme_sigstat_get_type(void)
 	    NULL,		/* class_data */
 	    sizeof(GMimeGpgmeSigstat),	/* instance_size */
 	    0,			/* n_preallocs */
-	    (GInstanceInitFunc) g_mime_gpgme_sigstat_init,	/* instance_init */
+	    NULL,	    /* instance_init */
 	    /* no value_table */
 	};
 
@@ -69,11 +77,14 @@ g_mime_gpgme_sigstat_get_type(void)
 
 
 GMimeGpgmeSigstat *
-g_mime_gpgme_sigstat_new(void)
+g_mime_gpgme_sigstat_new(gpgme_ctx_t ctx)
 {
+	GMimeGpgmeSigstat *result;
 
-    return
-	GMIME_GPGME_SIGSTAT(g_object_new(GMIME_TYPE_GPGME_SIGSTAT, NULL));
+	result = GMIME_GPGME_SIGSTAT(g_object_new(GMIME_TYPE_GPGME_SIGSTAT, NULL));
+	result->protocol = gpgme_get_protocol(ctx);
+	result->status = GPG_ERR_NOT_SIGNED;
+    return result;
 }
 
 
@@ -82,37 +93,95 @@ g_mime_gpgme_sigstat_new_from_gpgme_ctx(gpgme_ctx_t ctx)
 {
     GMimeGpgmeSigstat *sig_stat;
     gpgme_verify_result_t result;
-    gpgme_error_t err;
 
     g_return_val_if_fail(ctx, NULL);
-    if (!(sig_stat = g_mime_gpgme_sigstat_new()))
-	return NULL;
+    sig_stat = g_mime_gpgme_sigstat_new(ctx);
 
-    sig_stat->status = GPG_ERR_NOT_SIGNED;	/* no signature available */
-    sig_stat->protocol = gpgme_get_protocol(ctx);
-
-    /* try to retreive the result of a verify operation */
-    if (!(result = gpgme_op_verify_result(ctx))
-	|| result->signatures == NULL)
-	return sig_stat;
-
-    /* there is at least one signature */
-    sig_stat->fingerprint = g_strdup(result->signatures->fpr);
-    sig_stat->sign_time = result->signatures->timestamp;
-    sig_stat->status = gpgme_err_code(result->signatures->status);
-    sig_stat->validity = result->signatures->validity;
-
-    /* try to get the related key */
-    err = gpgme_get_key(ctx, sig_stat->fingerprint, &sig_stat->key, 0);
-    if (err != GPG_ERR_NO_ERROR) {
-    	gchar errbuf[4096];		/* should be large enough... */
-
-    	gpgme_strerror_r(err, errbuf, sizeof(errbuf));
-    	g_message("could not retrieve the key with fingerprint %s: %s: %s",
-    		sig_stat->fingerprint, gpgme_strsource(err), errbuf);
+    /* try to retrieve the result of a verify operation */
+    result = gpgme_op_verify_result(ctx);
+    if ((result != NULL) && (result->signatures != NULL)) {
+        /* there is at least one signature */
+        sig_stat->fingerprint = g_strdup(result->signatures->fpr);
+        sig_stat->sign_time = result->signatures->timestamp;
+        sig_stat->summary = result->signatures->summary;
+        sig_stat->status = gpgme_err_code(result->signatures->status);
+        sig_stat->validity = result->signatures->validity;
     }
 
-    return sig_stat;
+	return sig_stat;
+}
+
+void
+g_mime_gpgme_sigstat_load_key(GMimeGpgmeSigstat *sigstat)
+{
+	g_return_if_fail(GMIME_IS_GPGME_SIGSTAT(sigstat));
+
+	if ((sigstat->key == NULL) && ((sigstat->summary & GPGME_SIGSUM_KEY_MISSING) == 0)) {
+		gpgme_ctx_t ctx;
+
+		ctx = libbalsa_gpgme_new_with_proto(sigstat->protocol, NULL, NULL, NULL);
+		sigstat->key = libbalsa_gpgme_load_key(ctx, sigstat->fingerprint, NULL);
+		gpgme_release(ctx);
+	}
+}
+
+const gchar *
+g_mime_gpgme_sigstat_protocol_name(const GMimeGpgmeSigstat *sigstat)
+{
+	g_return_val_if_fail(GMIME_IS_GPGME_SIGSTAT(sigstat), NULL);
+
+    switch (sigstat->protocol) {
+    case GPGME_PROTOCOL_OpenPGP:
+    	return _("PGP signature: ");
+    case GPGME_PROTOCOL_CMS:
+    	return _("S/MIME signature: ");
+    default:
+    	return _("(unknown protocol) ");
+    }
+}
+
+static inline void
+append_time_t(GString     *str,
+			  const gchar *format,
+			  time_t       when,
+              const gchar *date_string)
+{
+    if (when != (time_t) 0) {
+        gchar *tbuf = libbalsa_date_to_utf8(when, date_string);
+        g_string_append_printf(str, format, tbuf);
+        g_free(tbuf);
+    } else {
+        g_string_append_printf(str, format, _("never"));
+    }
+}
+
+gchar *
+g_mime_gpgme_sigstat_to_gchar(const GMimeGpgmeSigstat *info,
+							  gboolean                 full_details,
+				 	 	 	  const gchar             *date_string)
+{
+    GString *msg;
+
+    g_return_val_if_fail(GMIME_IS_GPGME_SIGSTAT(info), NULL);
+    g_return_val_if_fail(date_string != NULL, NULL);
+    msg = g_string_new(g_mime_gpgme_sigstat_protocol_name(info));
+    msg = g_string_append(msg, libbalsa_gpgme_sig_stat_to_gchar(info->status));
+    g_string_append_printf(msg, _("\nSignature validity: %s"), libbalsa_gpgme_validity_to_gchar(info-> validity));
+    append_time_t(msg, _("\nSigned on: %s"), info->sign_time, date_string);
+    if (info->fingerprint) {
+    	g_string_append_printf(msg, _("\nKey fingerprint: %s"), info->fingerprint);
+    }
+
+    /* append key data */
+    if (full_details && (info->key != NULL)) {
+    	gchar *key_data;
+
+    	key_data = libbalsa_gpgme_key_to_gchar(info->key, info->fingerprint);
+    	g_string_append_printf(msg, "\n%s", key_data);
+    	g_free(key_data);
+    }
+
+    return g_string_free(msg, FALSE);
 }
 
 
@@ -225,11 +294,3 @@ libbalsa_cert_subject_readable(const gchar *subject)
     libbalsa_utf8_sanitize(&readable_subject, TRUE, NULL);
     return readable_subject;
 }
-
-static void
-g_mime_gpgme_sigstat_init(GMimeGpgmeSigstat * self)
-{
-    self->status = GPG_ERR_NOT_SIGNED;
-    self->key = NULL;
-    self->fingerprint = NULL;
-}
