@@ -33,10 +33,12 @@
 #  include "macosx-helpers.h"
 #endif
 
-#if ENABLE_ESMTP
+#ifdef HAVE_GPGME
+#  include "libbalsa-gpgme.h"
+#endif
+
 #include <string.h>
 #include "smtp-server.h"
-#endif                          /* ENABLE_ESMTP */
 
 /*
  * The class.
@@ -113,7 +115,8 @@ libbalsa_identity_init(LibBalsaIdentity* ident)
     ident->always_trust = FALSE;
     ident->warn_send_plain = TRUE;
     ident->crypt_protocol = LIBBALSA_PROTECT_OPENPGP;
-    ident->force_key_id = NULL;
+    ident->force_gpg_key_id = NULL;
+    ident->force_smime_key_id = NULL;
     ident->request_mdn = FALSE;
     ident->request_dsn = FALSE;
     /*
@@ -139,13 +142,12 @@ libbalsa_identity_finalize(GObject * object)
     g_free(ident->reply_string);
     g_free(ident->forward_string);
     g_free(ident->signature_path);
-#if ENABLE_ESMTP
     if (ident->smtp_server)
         g_object_unref(ident->smtp_server);
-#endif                          /* ENABLE_ESMTP */
     g_free(ident->face);
     g_free(ident->x_face);
-    g_free(ident->force_key_id);
+    g_free(ident->force_gpg_key_id);
+    g_free(ident->force_smime_key_id);
 
     G_OBJECT_CLASS(parent_class)->finalize(object);
 }
@@ -324,6 +326,7 @@ gchar*
 libbalsa_identity_get_signature(LibBalsaIdentity* identity, GtkWindow *parent)
 {
     gchar *ret = NULL, *path;
+    gchar *retval;
 
     if (identity->signature_path == NULL ||
         *identity->signature_path == '\0')
@@ -331,63 +334,49 @@ libbalsa_identity_get_signature(LibBalsaIdentity* identity, GtkWindow *parent)
 
     path = libbalsa_expand_path(identity->signature_path);
     if(identity->sig_executable){
-        FILE *fp;
+        GError *error = NULL;
+        gchar *argv[] = {"/bin/sh", "-c", path, NULL};
 
-        /* signature is executable */
-	fp = popen(path,"r");
-        if (fp) {
-            libbalsa_readfile_nostat(fp, &ret);
-            pclose(fp);
-        } else
-            libbalsa_information_parented
-                (parent, LIBBALSA_INFORMATION_ERROR,
-                 _("Error executing signature generator %s"),
-                 identity->signature_path);
-    } else {
-        FILE *fp;
-
-        /* sign is normal file */
-        fp = fopen(path, "r");
-        if (fp) {
-            libbalsa_readfile_nostat(fp, &ret);
-            fclose(fp);
-        } else
+        if (!g_spawn_sync(NULL, argv, NULL, G_SPAWN_DEFAULT, NULL, NULL,
+                          &ret, NULL, NULL, &error)) {
             libbalsa_information_parented(parent, LIBBALSA_INFORMATION_ERROR,
-                                          _("Cannot open signature file “%s” "
-                                            "for reading"),
-                                          identity->signature_path);
-    }
-    if(!ret)
-        libbalsa_information_parented(parent, LIBBALSA_INFORMATION_ERROR,
-                                   _("Error reading signature from %s"), path);
-    else {
-        if(!libbalsa_utf8_sanitize(&ret, FALSE, NULL))
-            libbalsa_information_parented
-                (parent, LIBBALSA_INFORMATION_ERROR,
-                 _("Signature in %s is not a UTF-8 text."),
-                 identity->signature_path);
+                                          _("Error executing signature generator “%s”: %s"),
+                                          identity->signature_path, error->message);
+            g_error_free(error);
+        }
+    } else {
+    	GError *error = NULL;
+
+    	if (!g_file_get_contents(path, &ret, NULL, &error)) {
+            libbalsa_information_parented(parent, LIBBALSA_INFORMATION_ERROR,
+                                          _("Cannot read signature file “%s”: %s"),
+                                          identity->signature_path, error->message);
+    		g_error_free(error);
+    	}
     }
     g_free(path);
 
     if(ret == NULL) return NULL;
 
+    if (!libbalsa_utf8_sanitize(&ret, FALSE, NULL)) {
+        libbalsa_information_parented(parent, LIBBALSA_INFORMATION_ERROR,
+                                      _("Signature in %s is not a UTF-8 text."),
+                                      identity->signature_path);
+    }
+
     /* Prepend the separator if needed... */
 
     if (identity->sig_separator
-        && strncmp(ret, "--\n", 3)
-        && strncmp(ret, "-- \n", 4)) {
-        gchar *sig_tmp = g_strconcat("\n-- \n", ret, NULL);
-        g_free(ret);
-        ret = sig_tmp;
+        && !(g_str_has_prefix(ret, "--\n") || g_str_has_prefix(ret, "-- \n"))) {
+        retval = g_strconcat("\n-- \n", ret, NULL);
     } else {
-        gchar *sig_tmp = g_strconcat("\n", ret, NULL);
-        g_free(ret);
-        ret = sig_tmp;
+        retval = g_strconcat("\n", ret, NULL);
     }
-    return ret;
+    g_free(ret);
+
+    return retval;
 }
 
-#if ENABLE_ESMTP
 void
 libbalsa_identity_set_smtp_server(LibBalsaIdentity * ident,
                                   LibBalsaSmtpServer * smtp_server)
@@ -400,7 +389,6 @@ libbalsa_identity_set_smtp_server(LibBalsaIdentity * ident,
     if (smtp_server)
 	g_object_ref(smtp_server);
 }
-#endif                          /* ENABLE_ESMTP */
 
 
 /* Used by both dialogs: */
@@ -565,12 +553,10 @@ sd_idle_add_response_ok(SelectDialogInfo * sdi)
 static gboolean
 sd_response_ok(SelectDialogInfo * sdi)
 {
-    gdk_threads_enter();
     if (sdi->idle_handler_id) {
         sdi->idle_handler_id = 0;
         gtk_dialog_response(GTK_DIALOG(sdi->dialog), GTK_RESPONSE_OK);
     }
-    gdk_threads_leave();
     return FALSE;
 }
 
@@ -602,6 +588,11 @@ static void ident_dialog_add_check_and_entry(GtkWidget *, gint, GtkDialog *,
                                              const gchar *, const gchar *);
 static void ident_dialog_add_entry(GtkWidget *, gint, GtkDialog *,
                                    const gchar *, const gchar *);
+static void ident_dialog_add_keysel_entry(GtkWidget   *grid,
+							  	  	  	  gint         row,
+										  GtkDialog   *dialog,
+										  const gchar *label_name,
+										  const gchar *entry_key);
 typedef enum LibBalsaIdentityPathType_ {
     LBI_PATH_TYPE_FACE,
     LBI_PATH_TYPE_XFACE
@@ -614,8 +605,6 @@ static void ident_dialog_add_file_chooser_button(GtkWidget * grid,
 static void ident_dialog_add_boxes(GtkWidget * grid, gint row,
                                    GtkDialog * dialog, const gchar * key1,
                                    const gchar * key2);
-#if ENABLE_ESMTP
-#endif /* ENABLE_ESMTP */
 static gchar *ident_dialog_get_text(GObject *, const gchar *);
 static gboolean ident_dialog_get_bool(GObject *, const gchar *);
 static gchar *ident_dialog_get_path(GObject * dialog, const gchar * key);
@@ -654,7 +643,6 @@ static void ident_dialog_free_values(GPtrArray * values);
 static void display_frame_set_gpg_mode(GObject * dialog,
                                        const gchar * key, gint * value);
 
-#if ENABLE_ESMTP
 static void ident_dialog_add_smtp_menu(GtkWidget * grid, gint row,
                                        GtkDialog * dialog,
                                        const gchar * label_name,
@@ -663,7 +651,6 @@ static void ident_dialog_add_smtp_menu(GtkWidget * grid, gint row,
 static void display_frame_set_server(GObject * dialog,
                                      const gchar * key,
                                      LibBalsaSmtpServer * smtp_server);
-#endif /* ENABLE_ESMTP */
 
 static gpointer ident_dialog_get_value(GObject * dialog,
                                        const gchar * key);
@@ -969,14 +956,9 @@ static const struct {
 static void md_sig_path_changed_cb(GtkToggleButton * sig_button,
                                    GObject * dialog);
 
-#if ENABLE_ESMTP
 static GtkWidget*
 setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree,
                   GSList * smtp_servers)
-#else                           /* ENABLE_ESMTP */
-static GtkWidget*
-setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
-#endif                          /* ENABLE_ESMTP */
 {
     GtkNotebook *notebook = GTK_NOTEBOOK(gtk_notebook_new());
     GtkWidget *grid;
@@ -1025,10 +1007,8 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
     ident_dialog_add_boxes(grid, row++, dialog,
                            path_info[LBI_PATH_TYPE_FACE].box_key,
                            path_info[LBI_PATH_TYPE_XFACE].box_key);
-#if ENABLE_ESMTP
     ident_dialog_add_smtp_menu(grid, row++, dialog, _("SMT_P server:"),
                                "identity-smtp-server", smtp_servers);
-#endif /* ENABLE_ESMTP */
 
     /* create the "Signature" tab */
     grid = append_ident_notebook_page(notebook, _("Signature"), NULL);
@@ -1075,15 +1055,19 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
 				 _("default protocol"),
 				 "identity-crypt-protocol");
     ident_dialog_add_checkbutton(grid, row++, dialog,
-                                 _("always trust GnuPG keys when encrypting"),
+                                 _("always trust GnuPG keys to encrypt messages"),
                                  "identity-trust-always", TRUE);
     ident_dialog_add_checkbutton(grid, row++, dialog,
                                  _("remind me if messages can be encrypted"),
                                  "identity-warn-send-plain", TRUE);
-    ident_dialog_add_entry(grid, row++, dialog,
-                           _("use secret key with this id for signing\n"
-                             "(leave empty for automatic selection)"),
-                           "identity-keyid");
+    ident_dialog_add_keysel_entry(grid, row++, dialog,
+                           	   	  _("use secret key with this id for signing GnuPG messages\n"
+                           	   		"(leave empty for automatic selection)"),
+								  "identity-keyid");
+    ident_dialog_add_keysel_entry(grid, row++, dialog,
+                           	   	  _("use certificate with this id for signing S/MIME messages\n"
+                           	   		"(leave empty for automatic selection)"),
+								  "identity-keyid-sm");
 #ifndef HAVE_GPGME
     gtk_widget_set_sensitive(grid, FALSE);
 #endif
@@ -1177,6 +1161,64 @@ ident_dialog_add_entry(GtkWidget * grid, gint row, GtkDialog * dialog,
         gtk_widget_grab_focus(entry);
 }
 
+
+#ifdef HAVE_GPGME
+static void
+choose_key(GtkButton *button, gpointer user_data)
+{
+	const gchar *target;
+	gpgme_protocol_t protocol;
+	gchar *email;
+	gchar *keyid;
+	GError *error = NULL;
+
+	target = g_object_get_data(G_OBJECT(button), "target");
+	if (strcmp(target, "identity-keyid") == 0) {
+		protocol = GPGME_PROTOCOL_OpenPGP;
+	} else {
+		protocol = GPGME_PROTOCOL_CMS;
+	}
+
+	email = ident_dialog_get_text(G_OBJECT(user_data), "identity-address");
+	keyid = libbalsa_gpgme_get_seckey(protocol, email, GTK_WINDOW(user_data), &error);
+	if (keyid != NULL) {
+		display_frame_set_field(G_OBJECT(user_data), target, keyid);
+		g_free(keyid);
+	}
+	if (error != NULL) {
+        libbalsa_information(LIBBALSA_INFORMATION_WARNING,
+                             _("Error selecting key: %s"), error->message);
+        g_clear_error(&error);
+	}
+}
+#endif
+
+
+/*
+ * Add a GtkEntry to the given dialog with a label next to it
+ * explaining the contents.  A reference to the entry is stored as
+ * object data attached to the dialog with the given key.  A button
+ * is added behind the entry to choose a key.
+ */
+static void
+ident_dialog_add_keysel_entry(GtkWidget   *grid,
+							  gint         row,
+							  GtkDialog   *dialog,
+                       	   	  const gchar *label_name,
+							  const gchar *entry_key)
+{
+	GtkWidget *button;
+
+	ident_dialog_add_entry(grid, row, dialog, label_name, entry_key);
+	button = gtk_button_new_with_label(_("Choose…"));
+#ifdef HAVE_GPGME
+	g_object_set_data_full(G_OBJECT(button), "target", g_strdup(entry_key), (GDestroyNotify) g_free);
+	g_signal_connect(button, "clicked", G_CALLBACK(choose_key), dialog);
+#endif
+	gtk_grid_attach(GTK_GRID(grid), button, 2, row, 1, 1);
+}
+
+
 /*
  * Add a GtkFileChooserButton to the given dialog with a label next to it
  * explaining the contents.  A reference to the button is stored as
@@ -1474,11 +1516,9 @@ ident_dialog_update(GObject * dlg)
     g_free(id->forward_string);
     id->forward_string  = ident_dialog_get_text(dlg, "identity-forwardstring");
     id->send_mp_alternative = ident_dialog_get_bool(dlg, "identity-sendmpalternative");
-#if ENABLE_ESMTP
     if(id->smtp_server) g_object_unref(id->smtp_server);
     id->smtp_server = ident_dialog_get_value(dlg, "identity-smtp-server");
     g_object_ref(id->smtp_server);
-#endif /* ENABLE_ESMTP */
 
     g_free(id->signature_path);
     id->signature_path  = ident_dialog_get_text(dlg, "identity-sigpath");
@@ -1503,8 +1543,10 @@ ident_dialog_update(GObject * dlg)
     id->warn_send_plain = ident_dialog_get_bool(dlg, "identity-warn-send-plain");
     id->crypt_protocol  = GPOINTER_TO_INT(ident_dialog_get_value
                                           (dlg, "identity-crypt-protocol"));
-    g_free(id->force_key_id);
-    id->force_key_id    = g_strstrip(ident_dialog_get_text(dlg, "identity-keyid"));
+    g_free(id->force_gpg_key_id);
+    id->force_gpg_key_id = g_strstrip(ident_dialog_get_text(dlg, "identity-keyid"));
+    g_free(id->force_smime_key_id);
+    id->force_smime_key_id = g_strstrip(ident_dialog_get_text(dlg, "identity-keyid-sm"));
 
     return TRUE;
 }
@@ -1670,12 +1712,18 @@ delete_ident_response(GtkWidget * confirm, gint response,
 static void
 help_ident_cb(GtkWidget * widget)
 {
-    GdkScreen *screen;
     GError *err = NULL;
+#if GTK_CHECK_VERSION(3, 22, 0)
+
+    gtk_show_uri_on_window(GTK_WINDOW(widget), "help:balsa/identities",
+                           gtk_get_current_event_time(), &err);
+#else /* GTK_CHECK_VERSION(3, 22, 0) */
+    GdkScreen *screen;
 
     screen = gtk_widget_get_screen(widget);
     gtk_show_uri(screen, "help:balsa/identities",
                  gtk_get_current_event_time(), &err);
+#endif /* GTK_CHECK_VERSION(3, 22, 0) */
 
     if (err) {
         g_print(_("Error displaying help for identities: %s\n"),
@@ -1689,7 +1737,6 @@ help_ident_cb(GtkWidget * widget)
    identites, the default one. Additionally, a callback is passed that
    will be executed when the identity list is modified: new entries
    are added or other entries are removed. */
-#if ENABLE_ESMTP
 static void
 lbi_free_smtp_server_list(GSList ** smtp_server_list)
 {
@@ -1697,15 +1744,11 @@ lbi_free_smtp_server_list(GSList ** smtp_server_list)
     g_slist_free(*smtp_server_list);
     g_free(smtp_server_list);
 }
-#endif /* ENABLE_ESMTP */
 
 void
 libbalsa_identity_config_dialog(GtkWindow *parent, GList **identities,
-				LibBalsaIdentity **default_id,
-#if ENABLE_ESMTP
-				GSList * smtp_servers,
-#endif /* ENABLE_ESMTP */
-                                void (*changed_cb)(gpointer))
+				LibBalsaIdentity **default_id, GSList * smtp_servers,
+				void (*changed_cb)(gpointer))
 {
     static GtkWidget *dialog = NULL;
     GtkWidget* frame;
@@ -1713,9 +1756,7 @@ libbalsa_identity_config_dialog(GtkWindow *parent, GList **identities,
     GtkWidget* hbox;
     GtkTreeView* tree;
     GtkTreeSelection *select;
-#if ENABLE_ESMTP
     GSList **smtp_server_list;
-#endif /* ENABLE_ESMTP */
 
     /* Show only one dialog at a time. */
     if (dialog) {
@@ -1728,10 +1769,11 @@ libbalsa_identity_config_dialog(GtkWindow *parent, GList **identities,
                                     parent, /* must NOT be modal */
                                     GTK_DIALOG_DESTROY_WITH_PARENT |
                                     libbalsa_dialog_flags(),
-                                    _("_Help"),   IDENTITY_RESPONSE_HELP,
-                                    _("_New"),    IDENTITY_RESPONSE_NEW,
-                                    _("_Remove"), IDENTITY_RESPONSE_REMOVE,
-                                    _("_Close"),  IDENTITY_RESPONSE_CLOSE,
+                                    _("_Help"),             IDENTITY_RESPONSE_HELP,
+                                    /* Translators: button "New" identity */
+                                    C_("identity", "_New"), IDENTITY_RESPONSE_NEW,
+                                    _("_Remove"),           IDENTITY_RESPONSE_REMOVE,
+                                    _("_Close"),            IDENTITY_RESPONSE_CLOSE,
                                     NULL);
 #if HAVE_MACOSX_DESKTOP
     libbalsa_macosx_menu_for_parent(dialog, parent);
@@ -1755,7 +1797,6 @@ libbalsa_identity_config_dialog(GtkWindow *parent, GList **identities,
 
     gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 0);
 
-#if ENABLE_ESMTP
     smtp_server_list = g_new(GSList *, 1);
     *smtp_server_list = g_slist_copy(smtp_servers);
     g_slist_foreach(smtp_servers, (GFunc) g_object_ref, NULL);
@@ -1764,9 +1805,6 @@ libbalsa_identity_config_dialog(GtkWindow *parent, GList **identities,
     g_object_weak_ref(G_OBJECT(display_frame),
 	              (GWeakNotify) lbi_free_smtp_server_list,
 		      smtp_server_list);
-#else  /* ENABLE_ESMTP */
-    display_frame = setup_ident_frame(GTK_DIALOG(dialog), FALSE, tree);
-#endif /* ENABLE_ESMTP */
 
     gtk_box_pack_start(GTK_BOX(hbox), display_frame, TRUE, TRUE, 0);
 
@@ -1849,10 +1887,8 @@ display_frame_update(GObject * dialog, LibBalsaIdentity* ident)
                             ident->forward_string);
     display_frame_set_boolean(dialog, "identity-sendmpalternative",
                               ident->send_mp_alternative);
-#if ENABLE_ESMTP
     display_frame_set_server(dialog, "identity-smtp-server",
                              ident->smtp_server);
-#endif /* ENABLE_ESMTP */
 
     display_frame_set_path(dialog, "identity-sigpath",
                            ident->signature_path, FALSE);
@@ -1894,7 +1930,8 @@ display_frame_update(GObject * dialog, LibBalsaIdentity* ident)
                               ident->warn_send_plain);
     display_frame_set_gpg_mode(dialog, "identity-crypt-protocol",
 			   &ident->crypt_protocol);
-    display_frame_set_field(dialog, "identity-keyid", ident->force_key_id);
+    display_frame_set_field(dialog, "identity-keyid", ident->force_gpg_key_id);
+    display_frame_set_field(dialog, "identity-keyid-sm", ident->force_smime_key_id);
 }
 
 
@@ -1995,7 +2032,8 @@ libbalsa_identity_new_config(const gchar* name)
     ident->always_trust = libbalsa_conf_get_bool("GpgTrustAlways");
     ident->warn_send_plain = libbalsa_conf_get_bool("GpgWarnSendPlain=true");
     ident->crypt_protocol = libbalsa_conf_get_int("CryptProtocol=16");
-    ident->force_key_id = libbalsa_conf_get_string("ForceKeyID");
+    ident->force_gpg_key_id = libbalsa_conf_get_string("ForceKeyID");
+    ident->force_smime_key_id = libbalsa_conf_get_string("ForceKeyIDSMime");
 
     return ident;
 }
@@ -2017,11 +2055,9 @@ libbalsa_identity_save(LibBalsaIdentity* ident, const gchar* group)
     libbalsa_conf_set_string("ReplyString", ident->reply_string);
     libbalsa_conf_set_string("ForwardString", ident->forward_string);
     libbalsa_conf_set_bool("SendMultipartAlternative", ident->send_mp_alternative);
-#if ENABLE_ESMTP
     libbalsa_conf_set_string("SmtpServer",
                              libbalsa_smtp_server_get_name(ident->
                                                            smtp_server));
-#endif                          /* ENABLE_ESMTP */
 
     libbalsa_conf_set_string("SignaturePath", ident->signature_path);
     libbalsa_conf_set_bool("SigExecutable", ident->sig_executable);
@@ -2042,7 +2078,8 @@ libbalsa_identity_save(LibBalsaIdentity* ident, const gchar* group)
     libbalsa_conf_set_bool("GpgTrustAlways", ident->always_trust);
     libbalsa_conf_set_bool("GpgWarnSendPlain", ident->warn_send_plain);
     libbalsa_conf_set_int("CryptProtocol", ident->crypt_protocol);
-    libbalsa_conf_set_string("ForceKeyID", ident->force_key_id);
+    libbalsa_conf_set_string("ForceKeyID", ident->force_gpg_key_id);
+    libbalsa_conf_set_string("ForceKeyIDSMime", ident->force_smime_key_id);
 
     libbalsa_conf_pop_group();
 }
@@ -2085,11 +2122,9 @@ display_frame_set_gpg_mode(GObject * dialog, const gchar* key, gint * value)
         case LIBBALSA_PROTECT_OPENPGP:
 	    gtk_combo_box_set_active(opt_menu, 1);
             break;
-#ifdef HAVE_SMIME
         case LIBBALSA_PROTECT_SMIMEV3:
 	    gtk_combo_box_set_active(opt_menu, 2);
             break;
-#endif
         case LIBBALSA_PROTECT_RFC3156:
         default:
 	    gtk_combo_box_set_active(opt_menu, 0);
@@ -2126,10 +2161,8 @@ ident_dialog_add_gpg_menu(GtkWidget * grid, gint row, GtkDialog * dialog,
                   GINT_TO_POINTER(LIBBALSA_PROTECT_RFC3156), opt_menu);
     add_show_menu(_("GnuPG OpenPGP mode"),
                   GINT_TO_POINTER(LIBBALSA_PROTECT_OPENPGP), opt_menu);
-#ifdef HAVE_SMIME
     add_show_menu(_("GpgSM S/MIME mode"),
                   GINT_TO_POINTER(LIBBALSA_PROTECT_SMIMEV3), opt_menu);
-#endif
 }
 
 /* add_show_menu: helper function */
@@ -2150,7 +2183,6 @@ ident_dialog_free_values(GPtrArray * values)
     g_ptr_array_free(values, TRUE);
 }
 
-#if ENABLE_ESMTP
 static void
 ident_dialog_add_smtp_menu(GtkWidget * grid, gint row, GtkDialog * dialog,
                            const gchar * label_name,
@@ -2196,8 +2228,6 @@ display_frame_set_server(GObject * dialog, const gchar * key,
     }
 }
 
-#endif                          /* ENABLE_ESMTP */
-
 /*
  * Get the value of the active option menu item
  */
@@ -2255,11 +2285,11 @@ libbalsa_identity_combo_box(GList       * identities,
                            2, ident,
                            -1);
 
-        if (active_name && strcmp(active_name, name) == 0)
-            gtk_combo_box_set_active_iter(GTK_COMBO_BOX(combo_box), &iter);
-
         g_free(from);
         g_free(name);
+
+        if (active_name != NULL && strcmp(active_name, ident->identity_name) == 0)
+            gtk_combo_box_set_active_iter(GTK_COMBO_BOX(combo_box), &iter);
     }
     g_object_unref(store);
 
