@@ -34,6 +34,7 @@
 #include "filter-funcs.h"
 #include "libbalsa-conf.h"
 #include "misc.h"
+#include "send.h"
 #include "server.h"
 #include "smtp-server.h"
 #include "save-restore.h"
@@ -61,9 +62,6 @@ ask_password_real(LibBalsaServer * server, LibBalsaMailbox * mbox)
 #if defined(HAVE_LIBSECRET)
     static const gchar *remember_password_message =
         N_("_Remember password in Secret Service");
-#elif defined (HAVE_GNOME_KEYRING)
-    static const gchar *remember_password_message =
-        N_("_Remember password in keyring");
 #else
     static const gchar *remember_password_message =
         N_("_Remember password");
@@ -138,10 +136,8 @@ static gboolean
 ask_passwd_idle(gpointer data)
 {
     AskPasswdData* apd = (AskPasswdData*)data;
-    gdk_threads_enter();
     apd->res = ask_password_real(apd->server, apd->mbox);
     apd->done = TRUE;
-    gdk_threads_leave();
     g_cond_signal(&apd->cond);
     return FALSE;
 }
@@ -251,16 +247,16 @@ ask_password(LibBalsaServer *server, LibBalsaMailbox *mbox)
 	return password;
 }
 
-#if ENABLE_ESMTP
-static void
-authapi_exit (void)
+
+/* Note: data indicates if the function shall be re-scheduled (NULL) or not (!= NULL) */
+static gboolean
+send_queued_messages_auto_cb(gpointer data)
 {
-    g_slist_foreach(balsa_app.smtp_servers, (GFunc) g_object_unref, NULL);
-    g_slist_free(balsa_app.smtp_servers);
-    balsa_app.smtp_servers = NULL;
-    auth_client_exit ();
+	g_debug("%s: %p", __func__, data);
+	libbalsa_process_queue(balsa_app.outbox, balsa_find_sentbox_by_url, balsa_app.smtp_servers, FALSE, NULL);
+    return (data == NULL);
 }
-#endif /* ESMTP */
+
 
 void
 balsa_app_init(void)
@@ -272,16 +268,7 @@ balsa_app_init(void)
     balsa_app.identities = NULL;
     balsa_app.current_ident = NULL;
     balsa_app.local_mail_directory = NULL;
-
-#if ENABLE_ESMTP
     balsa_app.smtp_servers = NULL;
-
-    /* Do what's needed at application level to allow libESMTP
-       to use authentication.  */
-    auth_client_init ();
-    atexit (authapi_exit);
-#endif
-
     balsa_app.inbox = NULL;
     balsa_app.inbox_input = NULL;
     balsa_app.outbox = NULL;
@@ -312,7 +299,8 @@ balsa_app_init(void)
     balsa_app.sw_maximized = FALSE;
 
     balsa_app.toolbar_wrap_button_text = TRUE;
-    balsa_app.pwindow_option = WHILERETR;
+    balsa_app.send_progress_dialog = TRUE;
+    balsa_app.recv_progress_dialog = TRUE;
     balsa_app.wordwrap = FALSE; /* default to format=flowed. */
     balsa_app.wraplength = 72;
     balsa_app.browse_wrap = FALSE; /* GtkTextView will wrap for us. */
@@ -415,6 +403,8 @@ balsa_app_init(void)
     /* Message filing */
     balsa_app.folder_mru=NULL;
     balsa_app.fcc_mru=NULL;
+
+    libbalsa_auto_send_init(send_queued_messages_auto_cb);
 }
 
 void
@@ -449,10 +439,10 @@ balsa_app_destroy(void)
     if(balsa_app.debug) g_print("balsa_app: Finished cleaning up.\n");
 }
 
-static gint
+static gboolean
 check_new_messages_auto_cb(gpointer data)
 {
-    check_new_messages_real(balsa_app.main_window, TYPE_BACKGROUND);
+    check_new_messages_real(balsa_app.main_window, TRUE);
 
     if (balsa_app.debug)
         fprintf(stderr, "Auto-checked for new messages…\n");
@@ -470,7 +460,7 @@ update_timer(gboolean update, guint minutes)
 
     balsa_app.check_mail_timer_id = update ?
         g_timeout_add_seconds(minutes * 60,
-                              (GSourceFunc) check_new_messages_auto_cb,
+                              check_new_messages_auto_cb,
                               NULL) : 0;
 }
 
@@ -538,8 +528,6 @@ balsa_open_mailbox_list(gchar ** urls)
 
     g_return_if_fail(urls != NULL);
 
-    gdk_threads_enter();
-
     for (tmp = urls; *tmp; ++tmp) {
         gchar **p;
 
@@ -554,8 +542,6 @@ balsa_open_mailbox_list(gchar ** urls)
     }
 
     g_strfreev(urls);
-
-    gdk_threads_leave();
 }
 
 void
@@ -604,16 +590,12 @@ balsa_find_mailbox(LibBalsaMailbox * mailbox)
 {
     BalsaFind bf;
 
-    gdk_threads_enter();
-
     bf.data = mailbox;
     bf.mbnode = NULL;
     if (balsa_app.mblist_tree_store)
         gtk_tree_model_foreach(GTK_TREE_MODEL(balsa_app.mblist_tree_store),
                                find_mailbox, &bf);
 
-    gdk_threads_leave();
-
     return bf.mbnode;
 }
 
@@ -642,10 +624,6 @@ BalsaMailboxNode *
 balsa_find_dir(LibBalsaServer *server, const gchar * path)
 {
     BalsaFind bf;
-    gboolean is_sub_thread = libbalsa_am_i_subthread();
-
-    if (is_sub_thread)
-	gdk_threads_enter();
 
     bf.data = path;
     bf.server = server;
@@ -653,9 +631,6 @@ balsa_find_dir(LibBalsaServer *server, const gchar * path)
     gtk_tree_model_foreach(GTK_TREE_MODEL(balsa_app.mblist_tree_store),
 			   (GtkTreeModelForeachFunc) find_path, &bf);
 
-    if (is_sub_thread)
-	gdk_threads_leave();
-
     return bf.mbnode;
 }
 
@@ -778,10 +753,6 @@ balsa_find_iter_by_data(GtkTreeIter * iter , gpointer data)
 
     /* We may call it from initial config, it's ok for
        mblist_tree_store not to exist. */
-#ifdef BALSA_DEBUG_THREADS
-    if (libbalsa_am_i_subthread())
-        g_warning("%s sub-thread!\n", __func__);
-#endif
     if(!balsa_app.mblist_tree_store)
         return FALSE;
 
