@@ -107,6 +107,8 @@ balsa_register_pixmaps(void)
 	{ BALSA_PIXMAP_SEND,            "mail-send" },
 	{ BALSA_PIXMAP_RECEIVE,         "stock_mail-receive" },
 	{ BALSA_PIXMAP_SEND_RECEIVE,    "mail-send-receive" },
+	{ BALSA_PIXMAP_QUEUE,           "mail-queue" },
+	{ BALSA_PIXMAP_SEND_QUEUED,		"balsa-send-queued" },
 	{ BALSA_PIXMAP_FORWARD,         "mail-forward" },
 	{ BALSA_PIXMAP_IDENTITY,        "stock_contact" },
 	{ BALSA_PIXMAP_CONTINUE,	"stock_mail" },
@@ -177,7 +179,6 @@ balsa_register_pixmaps(void)
 
     balsa_icon_table =
         g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
-    gtk_icon_theme_append_search_path(icon_theme, BALSA_STD_PREFIX "/share/icons");
     gtk_icon_theme_append_search_path(icon_theme, BALSA_DATA_PREFIX);
 
     for (i = 0; i < G_N_ELEMENTS(balsa_icons); i++)
@@ -222,10 +223,6 @@ balsa_register_pixbufs(GtkWidget * widget)
     guint i;
     GtkIconTheme *icon_theme = gtk_icon_theme_get_default();
 
-    gtk_icon_theme_append_search_path(icon_theme,
-                                      BALSA_STD_PREFIX "/share/icons");
-    gtk_icon_theme_append_search_path(icon_theme, BALSA_DATA_PREFIX);
-
     for (i = 0; i < G_N_ELEMENTS(icons); i++) {
         GdkPixbuf *pixbuf;
         GError *err = NULL;
