@@ -835,7 +835,9 @@ condition_dialog_response(GtkWidget * dialog, gint response,
 {
     LibBalsaCondition *new_cnd;
     GError *err = NULL;
+#if !GTK_CHECK_VERSION(3, 22, 0)
     GdkScreen *screen;
+#endif /* GTK_CHECK_VERSION(3, 22, 0) */
 
     switch (response) {
     case GTK_RESPONSE_OK:       /* OK button */
@@ -891,9 +893,15 @@ condition_dialog_response(GtkWidget * dialog, gint response,
         gtk_widget_hide(dialog);
         break;
     case GTK_RESPONSE_HELP:     /* Help button */
+#if GTK_CHECK_VERSION(3, 22, 0)
+        gtk_show_uri_on_window(GTK_WINDOW(dialog),
+                               "help:balsa/win-filters#win-condition",
+                               gtk_get_current_event_time(), &err);
+#else  /* GTK_CHECK_VERSION(3, 22, 0) */
         screen = gtk_widget_get_screen(dialog);
         gtk_show_uri(screen, "help:balsa/win-filters#win-condition",
                      gtk_get_current_event_time(), &err);
+#endif  /* GTK_CHECK_VERSION(3, 22, 0) */
 	if (err) {
 	    balsa_information_parented(GTK_WINDOW(dialog),
 		    LIBBALSA_INFORMATION_WARNING,
@@ -1452,7 +1460,9 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
     GtkTreeIter iter;
     gboolean valid;
     GError *err = NULL;
+#if !GTK_CHECK_VERSION(3, 22, 0)
     GdkScreen *screen;
+#endif /* GTK_CHECK_VERSION(3, 22, 0) */
 
     switch (response) {
     case GTK_RESPONSE_OK:       /* OK button */
@@ -1491,9 +1501,14 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
         break;
 
     case GTK_RESPONSE_HELP:     /* Help button */
+#if GTK_CHECK_VERSION(3, 22, 0)
+        gtk_show_uri_on_window(GTK_WINDOW(dialog), "help:balsa/win-filters",
+                                   gtk_get_current_event_time(), &err);
+#else  /* GTK_CHECK_VERSION(3, 22, 0) */
         screen = gtk_widget_get_screen(dialog);
         gtk_show_uri(screen, "help:balsa/win-filters",
                      gtk_get_current_event_time(), &err);
+#endif  /* GTK_CHECK_VERSION(3, 22, 0) */
 	if (err) {
 	    balsa_information_parented(GTK_WINDOW(dialog),
 		    LIBBALSA_INFORMATION_WARNING,
@@ -1649,7 +1664,7 @@ change_filter_name(gchar * old_name,gchar * new_name)
             if (new_name)
                 lst->data=g_strdup(new_name);
             else
-                new_filters_names=g_list_remove(new_filters_names,lst->data);
+                new_filters_names=g_list_delete_link(new_filters_names,lst);
             return;
         }
 	
@@ -1677,7 +1692,7 @@ change_filter_name(gchar * old_name,gchar * new_name)
 	    g_free(p->old_name);
 	    g_free(p);
             filters_names_changes=
-                g_list_remove(filters_names_changes,lst->data);
+                g_list_delete_link(filters_names_changes,lst);
             return;
         }
 
