@@ -91,7 +91,9 @@ static void
 folder_conf_response(GtkDialog * dialog, int response,
                      CommonDialogData * cdd)
 {
+#if !GTK_CHECK_VERSION(3, 22, 0)
     GdkScreen *screen;
+#endif /* GTK_CHECK_VERSION(3, 22, 0) */
     GError *err = NULL;
 
     /* If mbnode's parent gets rescanned, mbnode will be finalized,
@@ -102,9 +104,14 @@ folder_conf_response(GtkDialog * dialog, int response,
 	g_object_ref(cdd->mbnode);
     switch (response) {
     case GTK_RESPONSE_HELP:
+#if GTK_CHECK_VERSION(3, 22, 0)
+        gtk_show_uri_on_window(GTK_WINDOW(dialog), "help:balsa/folder-config",
+                               gtk_get_current_event_time(), &err);
+#else /* GTK_CHECK_VERSION(3, 22, 0) */
         screen = gtk_widget_get_screen(GTK_WIDGET(dialog));
         gtk_show_uri(screen, "help:balsa/folder-config",
                      gtk_get_current_event_time(), &err);
+#endif /* GTK_CHECK_VERSION(3, 22, 0) */
         if (err) {
             balsa_information(LIBBALSA_INFORMATION_WARNING,
                               _("Error displaying config help: %s\n"),
@@ -272,9 +279,6 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
 #if defined(HAVE_LIBSECRET)
     static const gchar *remember_password_message =
         N_("_Remember password in Secret Service");
-#elif defined (HAVE_GNOME_KEYRING)
-    static const gchar *remember_password_message =
-        N_("_Remember password in keyring");
 #else
     static const gchar *remember_password_message =
         N_("_Remember password");
@@ -839,7 +843,7 @@ folder_conf_imap_sub_node(BalsaMailboxNode * mn)
                                    sdd, row, sdd->old_folder, label);
 
     ++row;
-    label = libbalsa_create_grid_label(_("Host:"), grid, row);
+    (void) libbalsa_create_grid_label(_("Host:"), grid, row);
     sdd->host_label =
         gtk_label_new(sdd->mbnode && sdd->mbnode->server
                       ? sdd->mbnode->server->host : "");
@@ -848,7 +852,7 @@ folder_conf_imap_sub_node(BalsaMailboxNode * mn)
     gtk_grid_attach(GTK_GRID(grid), sdd->host_label, 1, row, 1, 1);
 
     ++row;
-    label = libbalsa_create_grid_label(_("Subfolder of:"), grid, row);
+    (void) libbalsa_create_grid_label(_("Subfolder of:"), grid, row);
     sdd->parent_folder = gtk_entry_new();
     gtk_editable_set_editable(GTK_EDITABLE(sdd->parent_folder), FALSE);
     gtk_entry_set_text(GTK_ENTRY(sdd->parent_folder), sdd->old_parent);
@@ -879,7 +883,7 @@ folder_conf_imap_sub_node(BalsaMailboxNode * mn)
         gchar * quotas;
 
         ++row;
-        label = libbalsa_create_grid_label(_("Permissions:"), grid, row);
+        (void) libbalsa_create_grid_label(_("Permissions:"), grid, row);
 
         /* mailbox closed: no detailed permissions available */
         if (!libbalsa_mailbox_imap_is_connected(LIBBALSA_MAILBOX_IMAP(mn->mailbox))) {
@@ -940,7 +944,7 @@ folder_conf_imap_sub_node(BalsaMailboxNode * mn)
         g_free(rights);
 
         ++row;
-        label = libbalsa_create_grid_label(_("Quota:"), grid, row);
+        (void) libbalsa_create_grid_label(_("Quota:"), grid, row);
 
         /* mailbox closed: no quota available */
         if (!libbalsa_mailbox_imap_is_connected(LIBBALSA_MAILBOX_IMAP(mn->mailbox)))
