@@ -866,11 +866,8 @@ bmbn_scan_children_idle(BalsaMailboxNode ** mbnode)
     GSList *list;
     GSList *l;
 
-    gdk_threads_enter();
-
     if (!*mbnode) {
         g_free(mbnode);
-        gdk_threads_leave();
         return FALSE;
     }
 
@@ -907,8 +904,6 @@ bmbn_scan_children_idle(BalsaMailboxNode ** mbnode)
     }
     g_free(mbnode);
 
-    gdk_threads_leave();
-
     return FALSE;
 }
 
@@ -1075,7 +1070,8 @@ balsa_mailbox_node_get_context_menu(BalsaMailboxNode * mbnode)
 		   G_CALLBACK(folder_conf_add_imap_sub_cb), NULL);
     gtk_widget_show(submenu);
     
-    menuitem = gtk_menu_item_new_with_mnemonic(_("_New"));
+    /* Translators: popup menu item "New" mailbox or folder */
+    menuitem = gtk_menu_item_new_with_mnemonic(C_("mailbox", "_New"));
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), submenu);
     gtk_widget_show(menuitem);
     
