@@ -37,11 +37,9 @@
 #include "filter-funcs.h"
 #include "mailbox-filter.h"
 #include "libbalsa-conf.h"
-#include "threads.h"
 
-#if ENABLE_ESMTP
 #include "smtp-server.h"
-#endif                          /* ENABLE_ESMTP */
+#include "send.h"
 
 #define FOLDER_SECTION_PREFIX "folder-"
 #define MAILBOX_SECTION_PREFIX "mailbox-"
@@ -370,28 +368,6 @@ gint config_folder_update(BalsaMailboxNode * mbnode)
     return res;
 }				/* config_folder_update */
 
-static void
-pop3_progress_notify(LibBalsaMailbox* mailbox, int msg_type, int prog, int tot,
-                     const char* msg)
-{
-    MailThreadMessage *message;
-
-    message = g_new(MailThreadMessage, 1);
-    message->message_type = msg_type;
-    memcpy(message->message_string, msg, strlen(msg) + 1);
-    message->num_bytes = prog;
-    message->tot_bytes = tot;
-
-    /* FIXME: There is potential for a timeout with 
-       * the server here, if we don't get the lock back
-       * soon enough.. But it prevents the main thread from
-       * blocking on the mutt_lock, andthe pipe filling up.
-       * This would give us a deadlock.
-     */
-    if (write(mail_thread_pipes[1], (void *) &message, sizeof(void *))
-        != sizeof(void *))
-        g_warning("pipe error");
-}
 
 /* Initialize the specified mailbox, creating the internal data
    structures which represent the mailbox. */
@@ -416,9 +392,6 @@ config_mailbox_init(const gchar * prefix)
     }
 
     if (LIBBALSA_IS_MAILBOX_POP3(mailbox)) {
-        g_signal_connect(G_OBJECT(mailbox),
-                         "progress-notify", G_CALLBACK(pop3_progress_notify),
-                         mailbox);
 	balsa_app.inbox_input =
             g_list_prepend(balsa_app.inbox_input,
                            balsa_mailbox_node_new_from_mailbox(mailbox));
@@ -480,13 +453,10 @@ config_folder_init(const gchar * prefix)
 static gboolean
 config_warning_idle(const gchar * text)
 {
-    gdk_threads_enter();
     balsa_information(LIBBALSA_INFORMATION_WARNING, "%s", text);
-    gdk_threads_leave();
     return FALSE;
 }
 
-#if ENABLE_ESMTP
 static gboolean
 config_load_smtp_server(const gchar * key, const gchar * value, gpointer data)
 {
@@ -502,7 +472,6 @@ config_load_smtp_server(const gchar * key, const gchar * value, gpointer data)
 
     return FALSE;
 }
-#endif                          /* ENABLE_ESMTP */
 
 static gboolean
 load_gtk_print_setting(const gchar * key, const gchar * value, gpointer data)
@@ -575,16 +544,15 @@ static gint
 config_global_load(void)
 {
     gboolean def_used;
-    guint tmp;
+    guint filter_mask;
     static gboolean new_user = FALSE;
 
     config_address_books_load();
-#if ENABLE_ESMTP
+
     /* Load SMTP servers before identities. */
     libbalsa_conf_foreach_group(SMTP_SERVER_SECTION_PREFIX,
 	                        config_load_smtp_server,
 	                        &balsa_app.smtp_servers);
-#endif                          /* ENABLE_ESMTP */
 
     /* We must load filters before mailboxes, because they refer to the filters list */
     config_filters_load();
@@ -674,17 +642,17 @@ config_global_load(void)
     g_free(balsa_app.date_string);
     balsa_app.date_string =
 	libbalsa_conf_get_string("DateFormat=" DEFAULT_DATE_FORMAT);
-    libbalsa_mailbox_date_format = balsa_app.date_string;
 
     /* ... Headers to show */
     balsa_app.shown_headers = d_get_gint("ShownHeaders", HEADERS_SELECTED);
 
     g_free(balsa_app.selected_headers);
     {                           /* scope */
-        gchar *tmp = libbalsa_conf_get_string("SelectedHeaders="
-                                             DEFAULT_SELECTED_HDRS);
-        balsa_app.selected_headers = g_ascii_strdown(tmp, -1);
-        g_free(tmp);
+        gchar *selected_headers =
+            libbalsa_conf_get_string("SelectedHeaders="
+                                     DEFAULT_SELECTED_HDRS);
+        balsa_app.selected_headers = g_ascii_strdown(selected_headers, -1);
+        g_free(selected_headers);
     }
 
     balsa_app.expand_tree = libbalsa_conf_get_bool("ExpandTree=false");
@@ -793,16 +761,17 @@ config_global_load(void)
     balsa_app.show_sos_bar =
         libbalsa_conf_get_bool("ShowSOSbar=true");
 
-    /* ... Progress Window Dialog */
-    balsa_app.pwindow_option = d_get_gint("ProgressWindow", WHILERETR);
+    /* ... Progress Window Dialogs */
+	balsa_app.recv_progress_dialog = libbalsa_conf_get_bool("ShowRecvProgressDlg=true");
+    balsa_app.send_progress_dialog = libbalsa_conf_get_bool("ShowSendProgressDlg=true");
 
     /* ... deleting messages: defaults enshrined here */
-    tmp = libbalsa_mailbox_get_filter(NULL);
+    filter_mask = libbalsa_mailbox_get_filter(NULL);
     if (libbalsa_conf_get_bool("HideDeleted=true"))
-	tmp |= (1 << 0);
+	filter_mask |= (1 << 0);
     else
-	tmp &= ~(1 << 0);
-    libbalsa_mailbox_set_filter(NULL, tmp);
+	filter_mask &= ~(1 << 0);
+    libbalsa_mailbox_set_filter(NULL, filter_mask);
 
     balsa_app.expunge_on_close =
         libbalsa_conf_get_bool("ExpungeOnClose=true");
@@ -872,9 +841,9 @@ config_global_load(void)
     /* Spelling options ... */
     libbalsa_conf_push_group("Spelling");
 
-#if HAVE_GSPELL || HAVE_GTKSPELL
     balsa_app.spell_check_lang =
         libbalsa_conf_get_string("SpellCheckLanguage");
+#if HAVE_GSPELL || HAVE_GTKSPELL
     balsa_app.spell_check_active =
         libbalsa_conf_get_bool_with_default("SpellCheckActive", &def_used);
     if (def_used)
@@ -935,7 +904,6 @@ config_global_load(void)
     /* Sending options ... */
     libbalsa_conf_push_group("Sending");
 
-#if ENABLE_ESMTP
     /* ... SMTP servers */
     if (!balsa_app.smtp_servers) {
 	/* Transition code */
@@ -960,26 +928,25 @@ config_global_load(void)
 
         passphrase = libbalsa_conf_private_get_string("ESMTPPassphrase");
 	if (passphrase) {
-            gchar* tmp = libbalsa_rot(passphrase);
+            gchar* passphrase_rot = libbalsa_rot(passphrase);
             g_free(passphrase); 
-            libbalsa_server_set_password(server, tmp);
-	    g_free(tmp);
+            libbalsa_server_set_password(server, passphrase_rot);
+	    g_free(passphrase_rot);
         }
 
         /* default set to "Use TLS if possible" */
 	server->tls_mode = libbalsa_conf_get_int("ESMTPTLSMode=1");
 
-#if HAVE_SMTP_TLS_CLIENT_CERTIFICATE
 	passphrase =
 	    libbalsa_conf_private_get_string("ESMTPCertificatePassphrase");
 	if (passphrase) {
-            gchar* tmp = libbalsa_rot(passphrase);
+            gchar* passphrase_rot = libbalsa_rot(passphrase);
             g_free(passphrase);
-	    libbalsa_smtp_server_set_cert_passphrase(smtp_server, tmp);
+	    g_free(server->cert_passphrase);
+	    server->cert_passphrase = passphrase_rot;
 	}
-#endif
     }
-#endif                          /* ENABLE_ESMTP */
+
     /* ... outgoing mail */
     balsa_app.wordwrap = libbalsa_conf_get_bool("WordWrap=false");
     balsa_app.wraplength = libbalsa_conf_get_int("WrapLength=72");
@@ -996,6 +963,13 @@ config_global_load(void)
 
     balsa_app.autoquote = 
 	libbalsa_conf_get_bool("AutoQuote=true");
+    balsa_app.send_mail_auto = libbalsa_conf_get_bool("AutoSend=true");
+    balsa_app.send_mail_timer = libbalsa_conf_get_int("AutoSendDelay=15");
+    if ((balsa_app.send_mail_timer < 1U) || (balsa_app.send_mail_timer > 120U)) {
+    	balsa_app.send_mail_timer = 15U;
+    }
+    libbalsa_auto_send_config(balsa_app.send_mail_auto, balsa_app.send_mail_timer);
+
     balsa_app.reply_strip_html = 
 	libbalsa_conf_get_bool("StripHtmlInReply=true");
     balsa_app.forward_attached = 
@@ -1173,9 +1147,7 @@ gint
 config_save(void)
 {
     gint i;
-#if ENABLE_ESMTP
     GSList *list;
-#endif                          /* ENABLE_ESMTP */
 
     config_address_books_save();
     config_identities_save();
@@ -1278,7 +1250,8 @@ config_save(void)
     libbalsa_conf_set_bool("ShowPreviewPane", balsa_app.previewpane);
     libbalsa_conf_set_bool("ShowMailboxList", balsa_app.show_mblist);
     libbalsa_conf_set_bool("ShowTabs", balsa_app.show_notebook_tabs);
-    libbalsa_conf_set_int("ProgressWindow", balsa_app.pwindow_option);
+	libbalsa_conf_set_bool("ShowRecvProgressDlg", balsa_app.recv_progress_dialog);
+    libbalsa_conf_set_bool("ShowSendProgressDlg", balsa_app.send_progress_dialog);
     libbalsa_conf_set_int("LayoutType",     balsa_app.layout_type);
     libbalsa_conf_set_bool("ViewMessageOnOpen", balsa_app.view_message_on_open);
     libbalsa_conf_set_bool("AskBeforeSelect", balsa_app.ask_before_select);
@@ -1351,9 +1324,9 @@ config_save(void)
     libbalsa_conf_remove_group("Spelling");
     libbalsa_conf_push_group("Spelling");
 
-#if HAVE_GSPELL || HAVE_GTKSPELL
     libbalsa_conf_set_string("SpellCheckLanguage",
                              balsa_app.spell_check_lang);
+#if HAVE_GSPELL || HAVE_GTKSPELL
     libbalsa_conf_set_bool("SpellCheckActive", 
                            balsa_app.spell_check_active);
 #else                           /* HAVE_GTKSPELL */
@@ -1404,7 +1377,6 @@ config_save(void)
     libbalsa_conf_pop_group();
 
     /* Sending options ... */
-#if ENABLE_ESMTP
     for (list = balsa_app.smtp_servers; list; list = list->next) {
         LibBalsaSmtpServer *smtp_server = LIBBALSA_SMTP_SERVER(list->data);
         gchar *group;
@@ -1417,7 +1389,6 @@ config_save(void)
         libbalsa_smtp_server_save_config(smtp_server);
         libbalsa_conf_pop_group();
     }
-#endif                          /* ENABLE_ESMTP */
 
     libbalsa_conf_remove_group("Sending");
     libbalsa_conf_private_remove_group("Sending");
@@ -1429,6 +1400,8 @@ config_save(void)
     libbalsa_conf_set_bool("ForwardAttached", balsa_app.forward_attached);
 
 	libbalsa_conf_set_int("AlwaysQueueSentMail", balsa_app.always_queue_sent_mail);
+    libbalsa_conf_set_bool("AutoSend", balsa_app.send_mail_auto);
+    libbalsa_conf_set_int("AutoSendDelay", balsa_app.send_mail_timer);
 	libbalsa_conf_set_int("CopyToSentbox", balsa_app.copy_to_sentbox);
     libbalsa_conf_pop_group();
 
@@ -1596,7 +1569,6 @@ config_address_books_save(void)
                    (GFunc) config_address_book_save, NULL);
 }
 
-#if ENABLE_ESMTP
 static LibBalsaSmtpServer *
 find_smtp_server_by_name(const gchar * name)
 {
@@ -1618,26 +1590,21 @@ find_smtp_server_by_name(const gchar * name)
     g_return_val_if_fail(balsa_app.smtp_servers, NULL);
     return LIBBALSA_SMTP_SERVER(balsa_app.smtp_servers->data);
 }
-#endif                          /* ESMTP */
 
 static gboolean
 config_identity_load(const gchar * key, const gchar * value, gpointer data)
 {
     const gchar *default_ident = data;
     LibBalsaIdentity *ident;
-#if ENABLE_ESMTP
     gchar *smtp_server_name;
-#endif                          /* ENABLE_ESMTP */
 
     libbalsa_conf_push_group(key);
     ident = libbalsa_identity_new_config(value);
-#if ENABLE_ESMTP
     smtp_server_name = libbalsa_conf_get_string("SmtpServer");
     libbalsa_identity_set_smtp_server(ident,
                                       find_smtp_server_by_name
                                       (smtp_server_name));
     g_free(smtp_server_name);
-#endif                          /* ENABLE_ESMTP */
     libbalsa_conf_pop_group();
     balsa_app.identities = g_list_prepend(balsa_app.identities, ident);
     if (g_ascii_strcasecmp(default_ident, ident->identity_name) == 0)
