@@ -50,9 +50,7 @@ struct BalsaToolbarModel_ {
     GArray          *current;
     BalsaToolbarType type;
     GtkToolbarStyle  style;
-#if HAVE_GNOME
     GSettings       *settings;
-#endif /* HAVE_GNOME */
 };
 
 enum {
@@ -72,12 +70,10 @@ balsa_toolbar_model_finalize(GObject * object)
         g_hash_table_destroy(model->legal);
         model->legal = NULL;
     }
-#if HAVE_GNOME
     if (model->settings) {
         g_object_unref(model->settings);
         model->settings = NULL;
     }
-#endif /* HAVE_GNOME */
     G_OBJECT_CLASS(parent_class)->finalize(object);
 }
 
@@ -157,7 +153,9 @@ button_data toolbar_buttons[]={
     {"document-print",            N_("Print"),           FALSE},
     {BALSA_PIXMAP_REQUEST_MDN,   N_("Request\nMDN"),    FALSE},
     {BALSA_PIXMAP_SEND,          N_("Send"),            TRUE},
+    {BALSA_PIXMAP_SEND_QUEUED,   N_("Send queued"),     TRUE},
     {BALSA_PIXMAP_SEND_RECEIVE,  N_("Exchange"),        FALSE},
+	{BALSA_PIXMAP_QUEUE,         N_("Queue"),           TRUE},
     {BALSA_PIXMAP_ATTACHMENT,    N_("Attach"),          TRUE},
     {"document-save",             N_("Save"),            TRUE},
     {BALSA_PIXMAP_IDENTITY,      N_("Identity"),        FALSE},
@@ -278,7 +276,6 @@ tm_save_model(BalsaToolbarModel * model)
     libbalsa_conf_pop_group();
 }
 
-#if HAVE_GNOME
 /* GSettings change_cb
  */
 static void
@@ -292,7 +289,6 @@ tm_gsettings_change_cb(GSettings   * settings,
         model->style == (GtkToolbarStyle) (-1))
         balsa_toolbar_model_changed(model);
 }
-#endif /* HAVE_GNOME */
 
 /* Create a BalsaToolbarModel structure.
  */
@@ -320,11 +316,9 @@ balsa_toolbar_model_new(BalsaToolbarType          type,
     balsa_toolbar_model_add_entries(model, entries, n_entries);
     tm_load_model(model);
 
-#if HAVE_GNOME
     model->settings = g_settings_new("org.gnome.desktop.interface");
     g_signal_connect(model->settings, "changed",
                      G_CALLBACK(tm_gsettings_change_cb), model);
-#endif /* HAVE_GNOME */
 
     return model;
 }
@@ -502,7 +496,6 @@ static GtkToolbarStyle
 tm_default_style(void)
 {
     GtkToolbarStyle default_style = GTK_TOOLBAR_BOTH;
-#if HAVE_GNOME
     GSettings *settings;
     gchar *str;
 
@@ -520,19 +513,10 @@ tm_default_style(void)
         g_free(str);
     }
     g_object_unref(settings);
-#endif /* HAVE_GNOME */
 
     return default_style;
 }
 
-static void
-tm_set_style(GtkWidget * toolbar, BalsaToolbarModel * model)
-{
-    gtk_toolbar_set_style(GTK_TOOLBAR(toolbar),
-                          model->style != (GtkToolbarStyle) (-1) ?
-                          model->style : tm_default_style());
-}
-
 /* Populate a model
  */
 #define BALSA_TOOLBAR_ACTION_MAP "balsa-toolbar-action-map"
@@ -589,6 +573,11 @@ tm_populate(GtkWidget * toolbar, BalsaToolbarModel * model)
         }
         gtk_toolbar_insert((GtkToolbar *) toolbar, item, -1);
     }
+
+    gtk_toolbar_set_style(GTK_TOOLBAR(toolbar),
+                          model->style != (GtkToolbarStyle) (-1) ?
+                          model->style : tm_default_style());
+
     gtk_widget_show_all(toolbar);
 }
 
@@ -599,7 +588,6 @@ tm_changed_cb(BalsaToolbarModel * model, GtkWidget * toolbar)
 {
     balsa_toolbar_remove_all(toolbar);
     tm_populate(toolbar, model);
-    tm_set_style(toolbar, model);
     tm_save_model(model);
 }
 
@@ -700,18 +688,22 @@ tm_popup_position_func(GtkMenu * menu, gint * x, gint * y,
 #endif                          /*GTK_CHECK_VERSION(3, 22, 0) */
 
 static gboolean
-tm_do_popup_menu(GtkWidget * toolbar, GdkEventButton * event,
-                 toolbar_info * info)
+tm_popup_context_menu_cb(GtkWidget    * toolbar,
+                         gint           x,
+                         gint           y,
+                         gint           button,
+                         toolbar_info * info)
 {
     GtkWidget *menu;
 #if !GTK_CHECK_VERSION(3, 22, 0)
-    int button, event_time;
+    int event_time;
 #endif                          /*GTK_CHECK_VERSION(3, 22, 0) */
     guint i;
     GSList *group = NULL;
-    GtkToolbarStyle default_style = tm_default_style();
+    GtkToolbarStyle default_style;
+    GdkEvent *event;
 
-    if (info->menu)
+    if (info->menu != NULL)
         return FALSE;
 
     info->menu = menu = gtk_menu_new();
@@ -742,6 +734,8 @@ tm_do_popup_menu(GtkWidget * toolbar, GdkEventButton * event,
                          info);
     }
 
+    default_style = tm_default_style();
+
     for (i = 0; i < G_N_ELEMENTS(tm_toolbar_options); i++) {
 
         if (!tm_toolbar_options[i].text)
@@ -795,28 +789,23 @@ tm_do_popup_menu(GtkWidget * toolbar, GdkEventButton * event,
     }
 
     gtk_widget_show_all(menu);
-
-#if GTK_CHECK_VERSION(3, 22, 0)
     gtk_menu_attach_to_widget(GTK_MENU(menu), toolbar, NULL);
-    if (event)
-        gtk_menu_popup_at_pointer(GTK_MENU(menu),
-                                  (GdkEvent *) event);
-    else
+
+    event = gtk_get_current_event();
+#if GTK_CHECK_VERSION(3, 22, 0)
+    if (event != NULL && event->type == GDK_BUTTON_PRESS) {
+        gtk_menu_popup_at_pointer(GTK_MENU(menu), event);
+    } else {
         gtk_menu_popup_at_widget(GTK_MENU(menu),
                                  GTK_WIDGET(toolbar),
-                                 GDK_GRAVITY_CENTER, GDK_GRAVITY_CENTER,
+                                 GDK_GRAVITY_NORTH,
+                                 GDK_GRAVITY_SOUTH,
                                  NULL);
-#else                           /*GTK_CHECK_VERSION(3, 22, 0) */
-    if (event) {
-        button = event->button;
-        event_time = event->time;
-    } else {
-        button = 0;
-        event_time = gtk_get_current_event_time();
     }
+#else                           /*GTK_CHECK_VERSION(3, 22, 0) */
+    event_time = gtk_get_current_event_time();
 
-    gtk_menu_attach_to_widget(GTK_MENU(menu), toolbar, NULL);
-    if (button)
+    if (button >= 0)
         gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button,
                        event_time);
     else
@@ -824,32 +813,12 @@ tm_do_popup_menu(GtkWidget * toolbar, GdkEventButton * event,
                        toolbar, button, event_time);
 #endif                          /*GTK_CHECK_VERSION(3, 22, 0) */
 
+    if (event != NULL)
+        gdk_event_free(event);
+
     return TRUE;
 }
 
-static gboolean
-tm_button_press_cb(GtkWidget * toolbar, GdkEventButton * event,
-                   toolbar_info * info)
-{
-    /* Ignore double-clicks and triple-clicks */
-    if (event->button == 3 && event->type == GDK_BUTTON_PRESS)
-        return tm_do_popup_menu(toolbar, event, info);
-
-    return FALSE;
-}
-
-static gboolean
-tm_popup_menu_cb(GtkWidget * toolbar, toolbar_info * info)
-{
-    return tm_do_popup_menu(toolbar, NULL, info);
-}
-
-static void
-tm_realize_cb(GtkWidget * toolbar, BalsaToolbarModel * model)
-{
-    tm_set_style(toolbar, model);
-}
-
 GtkWidget *balsa_toolbar_new(BalsaToolbarModel * model,
                              GActionMap        * action_map)
 {
@@ -867,14 +836,11 @@ GtkWidget *balsa_toolbar_new(BalsaToolbarModel * model,
     tm_populate(toolbar, model);
 
     g_signal_connect(model, "changed", G_CALLBACK(tm_changed_cb), toolbar);
-    g_signal_connect(toolbar, "realize", G_CALLBACK(tm_realize_cb), model);
     g_object_weak_ref(G_OBJECT(toolbar),
                       (GWeakNotify) tm_toolbar_weak_notify, info);
 
-    g_signal_connect(toolbar, "button-press-event",
-                     G_CALLBACK(tm_button_press_cb), info);
-    g_signal_connect(toolbar, "popup-menu", G_CALLBACK(tm_popup_menu_cb),
-                     info);
+    g_signal_connect(toolbar, "popup-context-menu",
+                     G_CALLBACK(tm_popup_context_menu_cb), info);
 
     gtk_widget_show_all(toolbar);
 
