@@ -362,7 +362,9 @@ current_selection_changed_cb(GtkTreeSelection * selection, ToolbarPage * page)
 static void
 tp_dialog_response_cb(GtkDialog * dialog, gint response, gpointer data)
 {
+#if !GTK_CHECK_VERSION(3, 22, 0)
     GdkScreen *screen;
+#endif /* GTK_CHECK_VERSION(3, 22, 0) */
     GError *err = NULL;
 
     switch (response) {
@@ -371,9 +373,14 @@ tp_dialog_response_cb(GtkDialog * dialog, gint response, gpointer data)
         gtk_widget_destroy(GTK_WIDGET(dialog));
         break;
     case GTK_RESPONSE_HELP:
+#if GTK_CHECK_VERSION(3, 22, 0)
+        gtk_show_uri_on_window(GTK_WINDOW(dialog), "help:balsa/toolbar-prefs",
+                               gtk_get_current_event_time(), &err);
+#else  /* GTK_CHECK_VERSION(3, 22, 0) */
         screen = gtk_widget_get_screen(GTK_WIDGET(dialog));
         gtk_show_uri(screen, "help:balsa/toolbar-prefs",
                      gtk_get_current_event_time(), &err);
+#endif /* GTK_CHECK_VERSION(3, 22, 0) */
         if (err) {
             balsa_information(LIBBALSA_INFORMATION_WARNING,
 		    _("Error displaying toolbar help: %s\n"),
@@ -782,8 +789,7 @@ tp_page_swap_rows(ToolbarPage * page, gboolean forward)
 {
     GtkTreeModel *model =
         gtk_tree_view_get_model(GTK_TREE_VIEW(page->current));
-    GtkTreeSelection *selection =
-        gtk_tree_view_get_selection(GTK_TREE_VIEW(page->current));
+    GtkTreeSelection *selection;
     GtkTreeIter iter;
     GtkTreeIter tmp_iter;
     GtkTreePath *tmp_path;
@@ -836,7 +842,7 @@ tp_store_set(GtkListStore * store, GtkTreeIter * iter, gint item)
     pixbuf =
         (item > 0
          ? gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
-                                    toolbar_buttons[item].pixmap_id,
+                                    balsa_icon_id(toolbar_buttons[item].pixmap_id),
                                     GTK_ICON_SIZE_LARGE_TOOLBAR, 0, NULL)
          : NULL);
     gtk_list_store_set(store, iter,
